@@ -34,6 +34,7 @@
 from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import Gd
+from gi.repository import Gio
 from gi.repository import Grl
 from gi.repository import Gio
 from gi.repository import Pango
@@ -90,8 +91,8 @@ class ViewContainer(Gtk.Stack):
         )
         self.view.set_view_type(view_type)
         self.view.set_model(self.model)
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        box.pack_start(self.view, True, True, 0)
+        self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        self._box.pack_start(self.view, True, True, 0)
         if use_sidebar:
             self.stack = Gtk.Stack(
                 transition_type=Gtk.StackTransitionType.SLIDE_RIGHT,
@@ -101,11 +102,11 @@ class ViewContainer(Gtk.Stack):
             if sidebar:
                 self.stack.add_named(sidebar, 'sidebar')
             else:
-                self.stack.add_named(box, 'sidebar')
+                self.stack.add_named(self._box, 'sidebar')
             self.stack.set_visible_child_name('dummy')
             self._grid.add(self.stack)
         if not use_sidebar or sidebar:
-            self._grid.add(box)
+            self._grid.add(self._box)
 
         self.star_handler = Widgets.StarHandler(self, 9)
         self.view.click_handler = self.view.connect('item-activated', self._on_item_activated)
@@ -266,12 +267,38 @@ class Empty(Gtk.Stack):
     def __init__(self, window, player):
         Gtk.Stack.__init__(self,
                            transition_type=Gtk.StackTransitionType.CROSSFADE)
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
-        widget = builder.get_object('container')
+        self.builder = Gtk.Builder()
+        self.builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
+        widget = self.builder.get_object('container')
+        self.update_empty_state_link()
         self.add(widget)
         self.show_all()
 
+    def update_empty_state_link(self):
+        label = self.builder.get_object('empty-state-label')
+        href_text = '<a href="%s">%s</a>' % (Query.MUSIC_URI, _("Music folder"))
+        label.set_label(label.get_label() % href_text)
+
+
+# Class for the Initial State
+class InitialState(Empty):
+    @log
+    def __init__(self, window, player):
+        Empty.__init__(self, window, player)
+
+        # Update image
+        icon = self.builder.get_object('icon')
+        icon.set_margin_bottom(32)
+        icon.set_opacity(1)
+        icon.set_from_resource('/org/gnome/Music/initial-state.png')
+        icon.set_size_request(256, 256)
+
+        # Update label
+        label = self.builder.get_object('label')
+        label.set_label(_("Hey DJ"))
+        label.set_opacity(1)
+        label.set_margin_bottom(18)
+
 
 class Albums(ViewContainer):
     @log
@@ -583,14 +610,19 @@ class Artists (ViewContainer):
         )
         self.artistAlbumsStack.add_named(self._artistAlbumsWidget, "sidebar")
         self.artistAlbumsStack.set_visible_child_name("sidebar")
-        self.view.set_shadow_type(Gtk.ShadowType.IN)
-        self.view.get_style_context().add_class('side-panel')
         self.view.set_hexpand(False)
         self.view.get_generic_view().get_selection().set_mode(
             Gtk.SelectionMode.SINGLE)
         self._grid.attach(self.artistAlbumsStack, 2, 0, 2, 2)
         self._add_list_renderers()
         self.view.get_generic_view().get_style_context().remove_class('content-view')
+        if (Gtk.Settings.get_default().get_property(
+                'gtk_application_prefer_dark_theme')):
+            self.view.get_generic_view().get_style_context().\
+                add_class('artist-panel-dark')
+        else:
+            self.view.get_generic_view().get_style_context().\
+                add_class('artist-panel-white')
         self.show_all()
         self.view.hide()
 
@@ -783,7 +815,9 @@ class Playlist(ViewContainer):
 
     @log
     def __init__(self, window, player):
-        self.playlists_sidebar = Gd.MainView()
+        self.playlists_sidebar = Gd.MainView(
+            shadow_type=Gtk.ShadowType.NONE
+        )
 
         ViewContainer.__init__(self, 'playlists', _("Playlists"), window,
                                Gd.MainViewType.LIST, True, self.playlists_sidebar)
@@ -825,7 +859,7 @@ class Playlist(ViewContainer):
         self.playlists_sidebar.set_view_type(Gd.MainViewType.LIST)
         self.playlists_sidebar.set_model(self.playlists_model)
         self.playlists_sidebar.set_hexpand(False)
-        self.playlists_sidebar.get_style_context().add_class('side-panel')
+        #self.playlists_sidebar.get_style_context().add_class('artist-panel')
         self.playlists_sidebar.get_generic_view().get_selection().set_mode(
             Gtk.SelectionMode.SINGLE)
         self.playlists_sidebar.connect('item-activated', self._on_playlist_activated)
@@ -834,6 +868,13 @@ class Playlist(ViewContainer):
         self._grid.child_set_property(self.stack, 'height', 2)
         self._add_sidebar_renderers()
         self.playlists_sidebar.get_generic_view().get_style_context().remove_class('content-view')
+        if (Gtk.Settings.get_default().get_property(
+                'gtk_application_prefer_dark_theme')):
+            self.playlists_sidebar.get_generic_view().get_style_context().\
+                add_class("artist-panel-dark")
+        else:
+            self.playlists_sidebar.get_generic_view().get_style_context().\
+                add_class("artist-panel-white")
 
         self.iter_to_clean = None
         self.iter_to_clean_model = None
@@ -1269,7 +1310,36 @@ class Playlist(ViewContainer):
                   for path in self.view.get_selection()])
 
 
+class EmptySearch(ViewContainer):
+    @log
+    def __init__(self, window, player):
+        ViewContainer.__init__(self, 'emptysearch', None, window, Gd.MainViewType.LIST)
+        self._artistAlbumsWidget = None
+        self.player = player
+
+        builder = Gtk.Builder()
+        builder.add_from_resource('/org/gnome/Music/NoMusic.ui')
+        widget = builder.get_object('container')
+        widget.set_vexpand(True)
+        widget.set_hexpand(True)
+        widget.get_children()[1].get_children()[1].set_text("Try a different search")
+        widget.show_all()
+        self._box.add(widget)
+
+    @log
+    def _back_button_clicked(self, widget, data=None):
+        self.header_bar.searchbar.show_bar(True, False)
+        if self.get_visible_child() == self._artistAlbumsWidget:
+            self._artistAlbumsWidget.destroy()
+            self._artistAlbumsWidget = None
+        self.set_visible_child(self._grid)
+
+
 class Search(ViewContainer):
+    __gsignals__ = {
+        'no-music-found': (GObject.SIGNAL_RUN_FIRST, None, ())
+    }
+
     @log
     def __init__(self, window, player):
         ViewContainer.__init__(self, 'search', None, window, Gd.MainViewType.LIST)
@@ -1286,6 +1356,8 @@ class Search(ViewContainer):
         self.player = player
         self.head_iters = [None, None, None, None]
         self.songs_model = self.model
+        self.previous_view = None
+        self.connect('no-music-found', self._no_music_found_callback)
 
         self.albums_selected = []
         self._albums = {}
@@ -1300,6 +1372,12 @@ class Search(ViewContainer):
         self.items_selected = []
         self.items_selected_callback = None
 
+    @log
+    def _no_music_found_callback(self, view):
+        self.window._stack.set_visible_child_name('emptysearch')
+        self.window._stack.get_child_by_name('emptysearch')._artistAlbumsWidget =\
+	        self._artistAlbumsWidget
+
     @log
     def _back_button_clicked(self, widget, data=None):
         self.header_bar.searchbar.show_bar(True, False)
@@ -1340,7 +1418,6 @@ class Search(ViewContainer):
                 self.header_bar, self.selection_toolbar, self.window, True
             )
             self.add(self._artistAlbumsWidget)
-            self._artistAlbumsWidget.show()
 
             self.header_bar.set_state(ToolbarState.SEARCH_VIEW)
             self.header_bar.header_bar.set_title(artist)
@@ -1392,10 +1469,21 @@ class Search(ViewContainer):
         if data is None:
             return
 
+        # We need to remember the view before the search view
+        if self.window.curr_view != self.window.views[5] and \
+           self.window.prev_view != self.window.views[5]:
+            self.previous_view = self.window.prev_view
+
         if remaining == 0:
             self.window.notification.dismiss()
             self.view.show()
 
+        if (self.model.iter_n_children(self.head_iters[0])+
+                self.model.iter_n_children(self.head_iters[1])+
+                self.model.iter_n_children(self.head_iters[2])+
+                self.model.iter_n_children(self.head_iters[3]) == 0) and remaining == 0:
+            self.emit('no-music-found')
+
         model, category = data
         if not item or model != self.model:
             return
