@@ -182,7 +182,7 @@ class Window(Gtk.ApplicationWindow):
     @log
     def _setup_view(self):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self.player = Player()
+        self.player = Player(self)
         self.selection_toolbar = SelectionToolbar()
         self.toolbar = Toolbar()
         self.views = []
@@ -237,13 +237,21 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _switch_to_empty_view(self):
-        self.views.append(Views.Empty(self, self.player))
+        did_initial_state = self.settings.get_boolean('did-initial-state')
+        view_class = None
+        if did_initial_state:
+            view_class = Views.Empty
+        else:
+            view_class = Views.InitialState
+        self.views.append(view_class(self, self.player))
+
         self._stack.add_titled(self.views[0], _("Empty"), _("Empty"))
         self.toolbar._search_button.set_sensitive(False)
         self.toolbar._select_button.set_sensitive(False)
 
     @log
     def _switch_to_player_view(self):
+        self.settings.set_boolean('did-initial-state', True)
         self._on_notify_model_id = self._stack.connect('notify::visible-child', self._on_notify_mode)
         self.connect('destroy', self._notify_mode_disconnect)
         self._key_press_event_id = self.connect('key_press_event', self._on_key_press)
@@ -253,6 +261,7 @@ class Window(Gtk.ApplicationWindow):
         self.views.append(Views.Songs(self, self.player))
         self.views.append(Views.Playlist(self, self.player))
         self.views.append(Views.Search(self, self.player))
+        self.views.append(Views.EmptySearch(self, self.player))
 
         for i in self.views:
             if i.title:
@@ -419,9 +428,15 @@ class Window(Gtk.ApplicationWindow):
            self.curr_view == self.views[3]:
             self.curr_view.stack.set_visible_child_name('dummy')
             self.curr_view.stack.set_visible_child_name('sidebar')
-        if self.curr_view != self.views[4]:
+        if self.curr_view != self.views[4] and self.curr_view != self.views[5]:
             self.toolbar.searchbar.show_bar(False)
 
+        # Toggle the selection button for the EmptySearch view
+        if self.curr_view == self.views[5] or \
+           self.prev_view == self.views[5]:
+            self.toolbar._select_button.set_sensitive(
+                not self.toolbar._select_button.get_sensitive())
+
     @log
     def _toggle_view(self, btn, i):
         self._stack.set_visible_child(self.views[i])
@@ -430,9 +445,11 @@ class Window(Gtk.ApplicationWindow):
     def _on_search_toggled(self, button, data=None):
         self.toolbar.searchbar.show_bar(button.get_active(),
                                         self.curr_view != self.views[4])
-        if not button.get_active() and self.curr_view == self.views[4] and \
+        if not button.get_active() and \
+           	(self.curr_view == self.views[4] or self.curr_view == self.views[5])and \
            self.toolbar._state == ToolbarState.MAIN:
-            self._stack.set_visible_child(self.prev_view)
+            # We should get back to the view before the search
+            self._stack.set_visible_child(self.views[4].previous_view)
             if self.toolbar._selectionMode:
                 self.toolbar.set_selection_mode(False)
 
